@@ -9,6 +9,7 @@ from abc import abstractmethod
 from collections import abc
 from enum import Enum
 from functools import wraps
+from pathlib import Path
 from typing import cast, ClassVar, Mapping, Union, Optional, Any, Callable, \
     Tuple
 from urllib.parse import urljoin, quote
@@ -302,13 +303,18 @@ class WSPipelines(BaseService):
         if url is not None:
             return url
 
+        name = 'OF_CPD_API_URL'
         url = os.environ.get('OF_CPD_API_URL', None)
-        if url is not None:
-            validate_type(url, "OF_CPD_API_URL", str)
-            return url
 
-        url = self.DEFAULT_CPD_API_URL
-        validate_type(url, "DEFAULT_CPD_API_URL", str)
+        if url is None:
+            name = 'RUNTIME_ENV_APSX_URL'
+            url = os.environ.get('RUNTIME_ENV_APSX_URL', None)
+
+        if url is None:
+            name = 'DEFAULT_CPD_API_URL'
+            url = self.DEFAULT_CPD_API_URL
+
+        validate_type(url, name, str)
         return url
 
     def get_scope(
@@ -484,11 +490,29 @@ class WSPipelines(BaseService):
 
          Mostly useful with zero arguments (to retrieve the default scope)
          or when handling ``Union[str, CpdScope]``."""
+        name = "cpd_scope"
+
         # if cpd_scope is None --- get it from env-var
+        def get_scope_from_env_var() -> Tuple[Optional[str], str]:
+            result = os.environ.get('OF_CPD_SCOPE', None)
+            if result is not None:
+                return result, 'OF_CPD_SCOPE'
+
+            result = os.environ.get('PROJECT_ID', None)
+            if result is not None:
+                return 'cpd:///projects/' + result, 'PROJECT_ID'
+
+            result = os.environ.get('SPACE_ID', None)
+            if result is not None:
+                return 'cpd:///spaces/' + result, 'SPACE_ID'
+
+            # default env-var
+            return None, 'OF_CPD_SCOPE'
+
         if cpd_scope is None:
-            cpd_scope = os.environ.get('OF_CPD_SCOPE', None)
+            cpd_scope, name = get_scope_from_env_var()
             if cpd_scope is None:
-                raise MissingValueError("OF_CPD_SCOPE")
+                raise MissingValueError(name)
 
         # if cpd_scope is str --- parse it
         if isinstance(cpd_scope, str):
@@ -498,7 +522,7 @@ class WSPipelines(BaseService):
                 raise OfCpdPathError(cpd_scope, reason = ex)
 
         # now it should be CpdScope
-        validate_type(cpd_scope, "OF_CPD_SCOPE", CpdScope)
+        validate_type(cpd_scope, name, CpdScope)
         return cpd_scope
 
     def store_results(
@@ -510,6 +534,19 @@ class WSPipelines(BaseService):
         for key, value in outputs.items():
             validate_type(key, f"outputs[...]", str)
 
+        test_mode = False
+        try:
+            output_artifacts = self.get_output_artifact_map()
+        except MissingValueError:
+            test_mode = True
+            output_artifacts = {
+                out: f'.ibm_watson_studio_pipelines/results/{out}'
+                for out in outputs.keys()
+            }
+
+        if test_mode:
+            print("Running outside of Watson Studio Pipeline - storing results in the local filesystem for testing purposes...")
+
         cpd_scope = self.get_scope_cpdpath() # needed for CPD variant anyway
         scope = self.get_scope(cpd_scope)
 
@@ -523,7 +560,12 @@ class WSPipelines(BaseService):
             guid = self._extract_storage_guid(scope)
             storage_client = CamsClient(self, cpd_scope, guid)
 
-        output_artifacts = self.get_output_artifact_map()
+        if test_mode:
+            print("")
+            print("  output paths:")
+            for out_name, out_path in output_artifacts.items():
+                print(f'    - "{out_name}": {out_path}')
+            storage_client = LocalFileSystemClient(self)
 
         response = None
         for output_name, output_value in outputs.items():
@@ -648,6 +690,45 @@ class StorageClient(Protocol):
     @abstractmethod
     def store_result(self, output_name: str, output_key: str, value: Any) -> DetailedResponse: ...
 
+class LocalFileSystemClient:
+    def __init__(
+        self,
+        cpd_orchestration: WSPipelines
+    ):
+        validate_type(cpd_orchestration, "cpd_orchestration", WSPipelines)
+        self.cpd_orchestration = cpd_orchestration
+
+    def store_result(self, output_name: str, output_key: str, value: Any) -> DetailedResponse:
+        validate_type(output_name, "output_name", str)
+        validate_type(output_key, "output_key", str)
+
+        if isinstance(value, io.TextIOBase):
+            # not supported yet
+            raise FilesResultsNotSupportedError(output_name)
+        else:
+            value = str(value)
+
+        path = Path(output_key)
+
+        status = 201 # created a new file
+        if path.exists():
+            status = 202 # updated an existing file
+
+        path.parent.mkdir(parents=True, exist_ok=True)
+        path.write_text(value)
+
+        response = requests.Response()
+        response.request = requests.Request(
+            method = 'PUT',
+            url = path.resolve().as_uri(),
+            headers = {},
+        )
+        response.status_code = status
+        response.headers = {}
+        return DetailedResponse(
+            response=response
+        )
+
 class CosClient:
     def __init__(
         self,
