@@ -203,9 +203,9 @@ class File:
 
 @attr.attrs(auto_attribs=True, frozen=True, kw_only=True)
 class Path:
-    scope: Optional[Segment[ScopeType]]
-    resource: Optional[Segment[Union[ResourceType, SpecialAssetsType]]]
-    file: Optional[File]
+    scope: Optional[Segment[ScopeType]] = None
+    resource: Optional[Segment[Union[ResourceType, SpecialAssetsType]]] = None
+    file: Optional[File] = None
 
     def __str__(self) -> str:
         result = ""
@@ -356,10 +356,10 @@ class Path:
 
 @attr.attrs(auto_attribs=True, frozen=True, kw_only=True)
 class CpdPathData:
-    has_prefix: bool
-    ctx: Optional[str]
+    has_prefix: bool = True
+    ctx: Optional[str] = None
     path: Path
-    query: Optional[Query]
+    query: Optional[Query] = None
 
     @classmethod
     def from_string(cls: Type[Self], path: str) -> Self:
@@ -570,6 +570,30 @@ class CpdPathImpl(CpdPath):
         return result
 
 class CpdScope(CpdPathImpl):
+    @classmethod
+    def project(cls: Type[Self], scope_id: str) -> Self:
+        data = CpdPathData(
+            path=Path(
+                scope=Segment(
+                    type=ScopeType.PROJECTS,
+                    id=scope_id,
+                )
+            )
+        )
+        return cls(data=data)
+
+    @classmethod
+    def space(cls: Type[Self], scope_id: str) -> Self:
+        data = CpdPathData(
+            path=Path(
+                scope=Segment(
+                    type=ScopeType.SPACES,
+                    id=scope_id,
+                )
+            )
+        )
+        return cls(data=data)
+
     # always present
     def scope_type(self) -> str:
         return self.data.path.scope.type.value
